#! CRADDON 1
#! NAME ChangeDateNew
#! DESCRIPTION Allows you to post items with a different date than the current one. Based on the official ChangeDate by Elvii, except this one supports Modify News and Time Offset. Works with newsdat.txt AND CorantoSQL! Now also has support for XMS Profiles (2.0 Beta 7)
#! VERSION 3.5.6
#! HOMEPAGE http://coranto.gweilo.org/
#! DOC 1

$::ChangeDateNew_build = 41;
$::ChangeDateNew_version = '3.5.6';


my $addon = new Addon('ChangeDateNew');
$addon->checkBuild(29);

if ($crcgiBuild >= 31) {
	$addon->isPrivacyCompatible();
}

# Check to see if the original ChangeDate addon is enabled. If it is, return a error message prompting users to disable it. Same goes for ChangeDateSQL.
if ($CConfig{'AddonsLoaded'} =~ /cra_changedate.pl/){
$addon->minorError("ChangeDateNew has determined that you have not disabled the official ChangeDate addon yet. Please disable this addon, then ChangeDate, and finally, re-enable this addon if you want to use it.");
}
if ($CConfig{'AddonsLoaded'} =~ /cra_changedatesql.pl/){
$addon->minorError("ChangeDateNew has determined that you have not disabled the ChangeDateSQL addon yet. Please disable this addon, then ChangeDate, and finally, re-enable this addon if you want to use it.");
}




my $ChangeDateNew_AdvSettings = <<'END_CODE';
# We have to do this because we don't want to load crcfg.pl & cradmin.pl unless accessing Advanced Settings
$addon->addAdvancedSettingHeading('ChangeDateNew Settings');
$CConfig{'ChangeDateNew_DateFormat'} = 'dd-mm-yyyy' unless $CConfig{'ChangeDateNew_DateFormat'};
$addon->addAdvancedSetting('ChangeDateNew_DateFormat', 'Date input format', 'Choose the format that you would like to use when entering dates.', ChangeDateNew_DateFormatSelect());
$addon->addAdvancedSetting('ChangeDateNew_FuturePost', 'Filter future items', 'If enabled, all items posted for a future date will not be included in Build News till the date has come or passed.','yn');
$addon->addAdvancedSetting('ChangeDateNew_UseOffset', 'Use Time Offset', 'If enabled, all items posted will have the Time Offset, that you selected in the Coranto settings, applied to them.','yn');
$addon->addAdvancedSetting('ChangeDateNew_TimeFilter', 'Ability to change both date and time', 'If enabled, the user will be able to change both date and time for the newsitem, otherwise only the submission date.','yn');
$addon->addAdvancedSetting('ChangeDateNew_EndDateFilter', 'Specify an enddate for individual news', 'If enabled, an extra datefield will appear in the submitpage where you can specify a date which will be the last day the news should be visible.','yn');
$CConfig{'changedatenew-profs'} = 'AllProfiles' unless $CConfig{'changedatenew-profs'};
$addon->addAdvancedSetting('changedatenew-profs', 'Active Enddate Profiles', 'Choose which Profiles you want the enddate to be active for. Only valid if you have choosed to enable enddate above. Multiple selections are allowed: to make multiple selections, Windows users hold down CTRL, Mac users hold down Option, most UNIX users hold down CTRL, users of other operating systems see your browser\'s help.', ChangeDateNew_ProfSelect());
if ($CConfig{'AddonsLoaded'} =~ /cra_xmultisubmit.pl/) {
	$CConfig{'changedatenew-xmsprofs'} = 'AllProfiles' unless $CConfig{'changedatenew-xmsprofs'};
	$addon->addAdvancedSetting('changedatenew-xmsprofs', 'XMS Profiles', 'Choose which Submit Pages you want ChangeDateNew to be shown on. Multiple selections are allowed: to make multiple selections, Windows users hold down CTRL, Mac users hold down Option, most UNIX users hold down CTRL, users of other operating systems see your browser\'s help.', ChangeDateNew_XMSProfSelect());
}
END_CODE

$addon->hook('AdvSettings', \$ChangeDateNew_AdvSettings);

sub ChangeDateNew_ProfSelect {
	my ($DisplayName, %profilesByDisplayName, %selectedprofs); 
	foreach $i (keys %newsprofiles) {
		# Do not present headline profiles in the selection box (2005-03-03)
		next if ($i =~ /(\S+)-headlines$/);
	    
		if (exists $newsprofiles{$i}->{'DisplayName'}){ 
			$DisplayName = $newsprofiles{$i}->{'DisplayName'}; 
		} 
		else { 
			$DisplayName = $i; 
		} 
		$profilesByDisplayName{lc $DisplayName}->{'realname'} = $i; 
	} 
          
	foreach my $i (split (/\(delim\)/, $CConfig{'changedatenew-profs'})) {
		$selectedprofs{$i} = 'selected';
	}

	my $catselect = qq~<select name="changedatenew-profs" size="6" multiple><option value="AllProfiles" $selectedprofs{AllProfiles}>(All Profiles)</option>~;
	foreach $DisplayName (sort keys %profilesByDisplayName) { 
		$i = $profilesByDisplayName{$DisplayName}->{'realname'};
		$catselect .= qq~<option value="$i" $selectedprofs{$i}>$DisplayName</option>~;
	}
	$catselect .= '</select>';
	return $catselect;
}


sub ChangeDateNew_SelectProfs {
	my $profname = shift;
	
	# If it is an headline profile, use the main settings instead (2005-03-03)
	$profname =~ s/(\S+)-headlines$/$1/;

	my (%ProfCats, $j);
	foreach $j (split (/\(delim\)/, $CConfig{'changedatenew-profs'})) {
		$ProfCats{$j} = 1;
	}

	return ($ProfCats{$profname} || $ProfCats{AllProfiles});
}



sub ChangeDateNew_XMSProfSelect {
	my %selectedxmsprofs;
	foreach my $i (split (/\(delim\)/, $CConfig{'changedatenew-xmsprofs'})) {
		$selectedxmsprofs{$i} = 'selected';
	}
	my $catselect = qq~<select name="changedatenew-xmsprofs" size="6" multiple><option value="AllProfiles" $selectedxmsprofs{AllProfiles}>(All Profiles)</option>~;
	
	XMS_LoadProf();
	foreach my $i(keys %::XMS_DB){
		if(exists $XMS_DB{$i}->{'Name'}){
			$catselect .= qq~<option value="$i" $selectedxmsprofs{$i}>$::XMS_DB{$i}->{'Name'}</option>~;
			}
		}
	$catselect .= '</select>';
	return $catselect;
}


sub ChangeDateNew_XMSSelectProfs {
	my (%ProfXMSCats, $profname, $i, $j);
	XMS_LoadProf();
		foreach my $i(keys %::XMS_DB){
			if(exists $XMS_DB{$i}->{'Name'}){
				$profname = $i if (($in{'profname'} eq $i) || ($::xMultiSubmitSP eq $i)) ;
            }
		}
			
	foreach $j (split (/\(delim\)/, $CConfig{'changedatenew-xmsprofs'})) {
		$ProfXMSCats{$j} = 1;
	}
	return ($ProfXMSCats{$profname} || $ProfXMSCats{AllProfiles});
}



sub ChangeDateNew_DateFormatSelect {
	# This is where the diferent formats are specified, if one is added here,
	# make sure that the subs ChangeDateNew_SplitDate and ChangeDateNew_VerifyDate is updated as well!
	my %CDN_DateFormats = ('yyyy-mm-dd'=>'yyyy-mm-dd', 'dd-mm-yyyy'=> 'dd-mm-yyyy', 'mm-dd-yyyy'=>'mm-dd-yyyy');

	return '<select name="ChangeDateNew_DateFormat">' . join('', map { qq~<option value="$_"~ . ( $CConfig{'ChangeDateNew_DateFormat'} eq $_ ? ' selected' : '' ) . qq~>$CDN_DateFormats{$_}</option>~ } sort { uc $CDN_DateFormats{$a} cmp uc $CDN_DateFormats{$b} } keys % CDN_DateFormats) . '</select>';
		
#	my %selectedDateFormat = ();
#	$selectedDateFormat{$CConfig{'ChangeDateNew_DateFormat'}} = 'selected';
	
#	my $catselect = qq~<select name="ChangeDateNew_DateFormat">~;
#	foreach my $i(sort { uc $CDN_DateFormats{$a} cmp uc $CDN_DateFormats{$b} } (keys %CDN_DateFormats)) {
#		$catselect .= qq~<option value="$i" $selectedDateFormat{$i}>$CDN_DateFormats{$i}</option>~;
#	}
#	$catselect .= '</select>';
	
#	return $catselect;
}
      





my $ChangeDateNew_DisplaySubForm_TopRow = <<'END_CODE';

	if ($CConfig{'AddonsLoaded'} =~ /cra_xmultisubmit.pl/) {
		if (ChangeDateNew_XMSSelectProfs()) {
			print FieldsRow("Alter submission date?<br>Optional ($CConfig{'ChangeDateNew_DateFormat'})", '<input type="text" name="ChangeDateNew_Date">');
			if($CConfig{'ChangeDateNew_TimeFilter'}) {
				print FieldsRow('Alter submission time?<br>Optional (hh:mm:ss)', '<input type="text" name="ChangeDateNew_Time">');
			}
            if ($CConfig{'ChangeDateNew_EndDateFilter'}) {
# PARAHEAD: Now using timestamps for ValidToDate instead (3.5)
#    			print FieldsRow("News valid until?<br>Optional ($CConfig{'ChangeDateNew_DateFormat'})", "<input type=\"text\" name=\"ChangeDateNew_ValidToDate\" value=\"$ChangeDateNew_ValidToDate\">");
    			print FieldsRow("News valid until?<br>Optional ($CConfig{'ChangeDateNew_DateFormat'})", "<input type=\"text\" name=\"ChangeDateNew_ValidToDate\" value=\"". ChangeDateNew_GetVisualDate($ChangeDateNew_ValidToDate) . "\">");
            }
		}
	}
	else {
		print FieldsRow("Alter submission date?<br>Optional ($CConfig{'ChangeDateNew_DateFormat'})", '<input type="text" name="ChangeDateNew_Date">');
		print FieldsRow('Alter submission time?<br>Optional (hh:mm:ss)', '<input type="text" name="ChangeDateNew_Time">');
        if ($CConfig{'ChangeDateNew_EndDateFilter'}) {
# PARAHEAD: Now using timestamps for ValidToDate instead (3.5)
#    			print FieldsRow("News valid until?<br>Optional ($CConfig{'ChangeDateNew_DateFormat'})", "<input type=\"text\" name=\"ChangeDateNew_ValidToDate\" value=\"$ChangeDateNew_ValidToDate\">");
				print FieldsRow("News valid until?<br>Optional ($CConfig{'ChangeDateNew_DateFormat'})", "<input type=\"text\" name=\"ChangeDateNew_ValidToDate\" value=\"". ChangeDateNew_GetVisualDate($ChangeDateNew_ValidToDate) . "\">");
        }
	}
END_CODE


#############################################################################################
# ADD SOME NEW HOOKS INTO THE CRA_NCATEGORY ADDON
#############################################################################################

my $ChangeDateNew_AddHooks = <<'END_CODE';

# Add an extra addon hook in the sub NewsCategories_EditCat 
$::Subs{'NewsCategories_EditCat'} =~ s/'\<div align="center"\>\<input type="submit"( class="inputsubmit")? value="Save Changes"\>\<\/div\>\<\/form\>';/
'';
if\(\$Addons\{'NewsCategories_EditCatBottom'\}\)\{my \$w;foreach \$w \(\@\{\$Addons\{'NewsCategories_EditCatBottom'\}\}\)\{my \$addon=\$w\-\>\[2\];eval \$\{\$w\-\>\[0\]\};AErr\(\$addon,\$@\)if \$@;\};\}
print '\<div align="center"\>\<input type="submit" class="inputsubmit" value="Save Changes"\>\<\/div\>\<\/form\>';/g;

# Add an extra addon hook in the sub NewsCategories_EditCatSave 
$::Subs{'NewsCategories_EditCatSave'} =~ s/NewsCategories_WriteCategoryInfo\(\);/
if\(\$Addons\{'NewsCategories_EditCatSave'\}\)\{my \$w;foreach \$w \(\@\{\$Addons\{'NewsCategories_EditCatSave'\}\}\)\{my \$addon=\$w\-\>\[2\];eval \$\{\$w\-\>\[0\]\};AErr\(\$addon,\$@\)if \$@;\};\}
NewsCategories_WriteCategoryInfo\(\);/g;

# Add an extra addon hook in the sub BuildNews
# Please note the extra block BUILDNEWS_SKIPDELETE, which makes it possible to skip the delete part...
$::Subs{'BuildNews'} =~ s/delete \$ActiveProfiles\{\$i\};(\s)+?next INITLOOP;/
BUILDNEWS_SKIPDELETE:{if\(\$Addons\{'BuildNews_Speedup_Code_Delete'\}\)\{my \$w;foreach \$w \(\@\{\$Addons\{'BuildNews_Speedup_Code_Delete'\}\}\)\{my \$addon=\$w\-\>\[2\];eval \$\{\$w\-\>\[0\]\};AErr\(\$addon,\$@\)if \$@;\};\}
delete \$ActiveProfiles\{\$i\};\n
next INITLOOP;}/g;

END_CODE
$addon->hook('EarlyHook', \$ChangeDateNew_AddHooks);



# We use this hook to add a new Change Date field to the Submit News page.
$addon->hook('DisplaySubForm_TopRow', \$ChangeDateNew_DisplaySubForm_TopRow, 10);

# We use this hook to add a new Change Date field to the Modify News Edit page.
$addon->hook('ModifyNews_Edit_TopRow', \$ChangeDateNew_DisplaySubForm_TopRow, 10);

#############################################################################################
# DON'T MODIFY ANYTHING BELOW THIS LINE!!
#############################################################################################

$addon->hook('SaveNews_1', 'ChangeDateNew_SaveNews_1', 10);
$addon->hook('ModifyNews_EditSave_Permissions', 'ChangeDateNew_SaveNews_1', 10);

$addon->hook('SaveNews_2', 'ChangeDateNew_SaveNews_2', -10);
$addon->hook('ModifyNews_EditSave_Done', 'ChangeDateNew_SaveNews_2', -10);

$addon->hook('ModifyNews_EditSave_Permissions', 'ChangeDateNew_UpdateNewstime', 9);

$addon->hook('NewsCategories_EditCatBottom', 'ChangeDateNew_EndDateCategory', 0);
$addon->hook('NewsCategories_EditCatSave', 'ChangeDateNew_EndDateCategory_Save', 0);
$addon->hook('BuildNews_Post', 'ChangeDateNew_EndDateCategory_Move', 0);





###########################################################################################
# THE SUBS ARE BELOW
###########################################################################################

# Verify that the entered date is OK, return a message and the date components
$Subs{'ChangeDateNew_VerifyDate'} = <<'END_SUB';
sub ChangeDateNew_VerifyDate {
	my $date = shift;
#	my ($date, $test) = @_;
	my ($Month_Day, $Month, $Year, $response);

#print "Date: $date<br>\n";		
	if($CConfig{'ChangeDateNew_DateFormat'} eq 'yyyy-mm-dd') {
		unless ($date =~ /^(\d\d\d\d)-(\d\d)-(\d\d)$/){
			$response = qq~The news date you have entered is incorrect, you must use the yyyy-mm-dd format.~;
		}
		($Year, $Month, $Month_Day) = ($1, $2, $3);
	} elsif($CConfig{'ChangeDateNew_DateFormat'} eq 'mm-dd-yyyy') {
		unless ($date =~ /^(\d\d)-(\d\d)-(\d\d\d\d)$/){
			$response = qq~The news date you have entered is incorrect, you must use the mm-dd-yyyy format.~;
		}
		($Year, $Month, $Month_Day) = ($3, $1, $2);
	} else {
		# Previously only dd-mm-yyyy existed, so make it the default
		unless ($date =~ /^(\d\d)-(\d\d)-(\d\d\d\d)$/){
			$response = qq~The news date you have entered is incorrect, you must use the dd-mm-yyyy format.~;
		}
		($Year, $Month, $Month_Day) = ($3, $2, $1);
	}

	unless($response) {
#		$Month--;
#		$Year -= 1900;
		$response = qq~The date '$date' is in error, Day must be in range 1..31, please go back and correct it~ if( ($Month_Day<1) || ($Month_Day>31) );
		$response = qq~The date '$date' is in error, Month must be in range 1..12, please go back and correct it~ if( ($Month<1) || ($Month>12) );
		$response = qq~The date '$date' is in error, Year must be in range 1900..2100, please go back and correct it~ if( ($Year<1900) || ($Year>2100) );
	}

#print "ChangeDateNew_VerifyDate: year: $Year, month: $Month, MDay: $Month_Day<br>\n";
		
	return ($response, $Year, $Month, $Month_Day);
}
END_SUB

# Splits the date into its components
$Subs{'ChangeDateNew_SplitDate'} = <<'END_SUB';
sub ChangeDateNew_SplitDate {
	my $date = shift;
	my ($dd, $mm, $yyyy);
	
	if($CConfig{'ChangeDateNew_DateFormat'} eq 'yyyy-mm-dd') {
		($yyyy, $mm, $dd) = split("-",$date);
	} elsif($CConfig{'ChangeDateNew_DateFormat'} eq 'mm-dd-yyyy') {
		($mm, $dd, $yyyy) = split("-",$date);
	} else {
		# Previously only dd-mm-yyyy existed, so make it the default
		($dd, $mm, $yyyy) = split("-",$date);
	}
	return ($yyyy, $mm, $dd);
}
END_SUB


# Takes a timestamp and returns the visual representation according to the current date format
$Subs{'ChangeDateNew_GetVisualDate'} = <<'END_SUB';
sub ChangeDateNew_GetVisualDate {
	my $date = shift;
	return '' unless($date);
	
	# PARAHEAD: This is only needed if we dont loop through the newsdatabase once first... (3.5)
	if($date =~ /-/) {
		my ($message, $yyyy, $mm, $dd) = ChangeDateNew_VerifyDate($date);
		if($message) {
			return '';
		} else {
			eval {
				$date = YMDtoUNIX($yyyy, $mm, $dd);
			};
			if ($@) {
				$addon->minorError("The ChangeDateNew addon (in sub ChangeDateNew_GetVisualDate) had problems converting ($Valid_Year, $Valid_Month, $Valid_MDay) via YMDtoUNIX: $@");
			}
		}
	}

	my ($VisualDate);
	
	eval {
		($Second,$Minute,$Hour,$Month_Day, $Month,$Year,$Week_Day,$IsDST) = localtime($date);	
	};
	if ($@) {
		$addon->minorError("ChangeDateNew could not convert the timestamp '$date' into a visual date format: $@");
	}

	$Month++;
	$Year += 1900;
	$Month_Day = "0$Month_Day" if $Month_Day < 10;
	$Month = "0$Month" if $Month < 10;
	
	if($CConfig{'ChangeDateNew_DateFormat'} eq 'yyyy-mm-dd') {
		$VisualDate = "$Year-$Month-$Month_Day";
	} elsif($CConfig{'ChangeDateNew_DateFormat'} eq 'mm-dd-yyyy') {
		$VisualDate = "$Month-$Month_Day-$Year";
	} else {
		# Previously only dd-mm-yyyy existed, so make it the default
		$VisualDate = "$Month_Day-$Month-$Year";
	}
	return $VisualDate;
}
END_SUB






$Subs{'ChangeDateNew_SaveNews_1'} = <<'END_SUB';
sub ChangeDateNew_SaveNews_1 {
my $addon = shift;

#&BasicDateVars($newstime);
#if ($in{'action'} ne 'modify-editsave'){
#$newstime = $CurrentTime;
#}

#my($Second,$Minute,$Hour,$Month_Day, $Month,$Year,$Week_Day,$IsDST, $Valid_Year, $Valid_Month, $Valid_MDay, $message);
my($Valid_Year, $Valid_Month, $Valid_MDay, $message);

	eval {
		# This is to see if localtime() is causing the "interal server error" problem
		($Second,$Minute,$Hour,$Month_Day, $Month,$Year,$Week_Day,$IsDST) = localtime($newstime);
		$Month++;
		$Year += 1900;	
#		($Second,$Minute,$Hour,$Month_Day, $Month,$Year,$Week_Day,$IsDST) = localtime(time);	
	};
	if ($@) {
##			$addon->fatalError($@);
$addon->minorError("ChangeDateNew depends on the method 'localtime' which caused an error: $@");
	}

	# PARAHEAD: Now using ValidToDate as timestamp (3.5)
	
#print "VT-Date: $in{'ChangeDateNew_ValidToDate'}<br>\n";
	$ChangeDateNew_ValidToDate = '';
	if($in{'ChangeDateNew_ValidToDate'}){
		($message, $Valid_Year, $Valid_Month, $Valid_MDay) = ChangeDateNew_VerifyDate($in{'ChangeDateNew_ValidToDate'});
		CRcough($message) if $message;
		eval {
			$ChangeDateNew_ValidToDate = YMDtoUNIX($Valid_Year, $Valid_Month, $Valid_MDay);
		};
		if ($@) {
			$addon->minorError("The ChangeDateNew addon (in sub ChangeDateNew_SaveNews_1) had problems with converting ($Valid_Year, $Valid_Month, $Valid_MDay) via YMDtoUNIX: $@");
		}
		
#		$ChangeDateNew_ValidToDate = YMDtoUNIX(2004, 12, 24);
#print "VT-Timestamp: $ChangeDateNew_ValidToDate<br>\n";
	}

	if($in{'ChangeDateNew_Date'} || $in{'ChangeDateNew_Time'}){

		if(!$CConfig{'ChangeDateNew_TimeFilter'}) {
			($Hour,$Minute,$Second) = split(":", "00:00:01");
		}
		elsif ($in{'ChangeDateNew_Time'}){
			unless ($in{'ChangeDateNew_Time'} =~ /^(\d\d):(\d\d):(\d\d)$/){
				CRcough(qq~The news time you have entered is incorrect, you must use the Hour:Minute:Seconds format. For example, 02:34:00~);
			}
			# Use the Hour, Minute and Time the user has set
			($Hour,$Minute,$Second) = split(":",$in{'ChangeDateNew_Time'});
		}
	
		if ($in{'ChangeDateNew_Date'}){
			($message, $Year, $Month, $Month_Day) = ChangeDateNew_VerifyDate($in{'ChangeDateNew_Date'});
			CRcough($message) if $message;
		}


		eval {
			# Now we use the new variables to change the time
			$newstime = timelocal($Second,$Minute,$Hour, $Month_Day, $Month-1, $Year-1900);
		};
		if ($@) {
#			$addon->minorError("ChangeDateNew, timelocal caused an error: $@");
#			$addon->simplePage("ChangeDateNew, timelocal caused an error: $@");
CRcough("The ChangeDateNew addon (in sub ChangeDateNew_SaveNews_1) had problems with timelocal causing an error: $@");
		}

		#print qq~newstime: $newstime~;

		# Finally, to account for Time Offset
		if ($CConfig{'TimeOffset'} && $CConfig{'ChangeDateNew_UseOffset'} == 1 && $in{'ChangeDateNew_Time'}){
			$newstime += (3600 * $CConfig{'TimeOffset'});
		}

		#print qq~offsetted newstime: $newstime~;
	}	

	$CConfig{'ChangeDateNew_FreshestTimestamp'} = $newstime if($newstime > $CConfig{'ChangeDateNew_FreshestTimestamp'});
	
	NeedCFG();
	unless (grep /^ChangeDateNew_ValidToDate$/, @fieldDB_internalorder) {
		push (@fieldDB_internalorder, 'ChangeDateNew_ValidToDate');
		NeedFile('cradmin.pl');
		SaveCRCFG();
	}
#	$ChangeDateNew_ValidToDate = $in{'ChangeDateNew_ValidToDate'};

}
END_SUB

$Subs{'ChangeDateNew_SaveNews_2'} = <<'END_SUB';
sub ChangeDateNew_SaveNews_2 {
my $addon = shift;

	unless ($CConfig{'AddonsLoaded'} =~ /cra_crsql/ && $CConfig{'CorantoSQL_built'}){
#		if ($in{'ChangeDateNew_Date'} || $in{'ChangeDateNew_Time'}) {
		if ($in{'ChangeDateNew_Date'} || $in{'ChangeDateNew_Time'} || ($newstime < $CConfig{'ChangeDateNew_FreshestTimestamp'}) ) {
			# Load news database.
			loadND();
			# Generate array of dates.
			my @ntime = map ($_->{'newstime'}, @NewsData);
			# Sort!
			@NewsData = @NewsData [ sort { $ntime[$b] <=> $ntime[$a] } 0 .. $#ntime];
			$CConfig{'ChangeDateNew_FreshestTimestamp'} = @NewsData[0]->{'newstime'};
			# Save!
			saveND();
			# The End
		}
	}

$CConfig{'LastBuildOverride'} = ($newstime - 1) unless $CConfig{'LastBuildOverride'} < $newstime;
}
END_SUB

$Subs{'ChangeDateNew_UpdateNewstime'} = <<'END_SUB';
sub ChangeDateNew_UpdateNewstime {
my $addon = shift;

	if ($in{'ChangeDateNew_Date'} || $in{'ChangeDateNew_Time'}){
		if ($CConfig{'AddonsLoaded'} =~ /cra_crsql.pl/){
		# Ok, we are using CorantoSQL, so we must UPDATE the table
		my $corantosql_dbh = &CorantoSQL_connectdb();
		my ($query,$sth);
		my $id = $in{'nid'};

			eval {
			$query= qq~UPDATE $CConfig{'CorantoSQL_tblname'} SET newstime=$newstime WHERE newsid=?~;
			#print qq~SQL Query: $query<br>~;
			$sth = $corantosql_dbh->prepare($query);
			$sth->execute($id);
			$sth->finish();
			};
			if ($@){
			$addon->minorError("Couldn't update the 'newstime' for the item you just submitted. Error message: $@");
			}

		$corantosql_dbh->disconnect();
		}
		else {
		# We are using newsdat.txt, so we don't have to do anything!!
		# It'll automatically use the $newstime var we set in &ChangeDateNew_SaveNews_1()
		}
	}
}
END_SUB



$Subs{'ChangeDateNew_EndDateCategory'} = <<'END_SUB';
sub ChangeDateNew_EndDateCategory {
	my $addon = shift;
	my $cat = $in{'cat'};
	my $nc = $newscategories{$cat};

	my $userselect = '<select name="changedatenew_cat"><option value="cdn_maintain" ' .
		( ($nc->{'changedatenew_cat'} eq 'cdn_maintain')  ? 'selected' : '') . '>Maintain Same Category</option>';
	foreach my $i (sort keys %newscategories) {
		$userselect .= qq~<option value="$i"~ . 
			( ($nc->{'changedatenew_cat'} eq $i) ? ' selected' : '') .
			">$i</option>";
	}
	$userselect .= '</select>';

	print MidHeading('Change Date New Settings');

	print $addon->settingTable('End Date Move:', $userselect, 
		qq~When the End Date (specified using the ChangeDateNew addon) is reached for a newsitem, which
		Category should the newsitem be transfered to instead. Note that the End Date field is cleared
		(set to empty) when the newsitem is moved to the new Category.~);
}
END_SUB



$Subs{'ChangeDateNew_EndDateCategory_Save'} = <<'END_SUB';
sub ChangeDateNew_EndDateCategory_Save {
	my $addon = shift;

	my $cat = $in{'cat'};
	$newscategories{$cat}->{'changedatenew_cat'} = $in{'changedatenew_cat'};
}
END_SUB



$Subs{'ChangeDateNew_EndDateCategory_Move'} = <<'END_SUB';
sub ChangeDateNew_EndDateCategory_Move {
	my $addon = shift;

	if (%ChangeDateNew_EndDateItems) {
		my ($fh, $fh2) = EditNewsdat_Start();

		while (<$fh>) {
			# Loop through items.
			chomp $_;
			SplitDataFile($_);
			foreach my $nid(keys %ChangeDateNew_EndDateItems) {
				if ($nid eq $newsid) {
					my $newCat = $newscategories{$Category}->{'changedatenew_cat'};
					if($newCat && ($newCat ne 'cdn_maintain')) {
						$Category = $newCat;
						$ChangeDateNew_ValidToDate = '';
					}
					delete $ChangeDateNew_EndDateItems{$nid};
				}
			}
#			last unless(%ChangeDateNew_EndDateItems);
			my $newsline = JoinDataFile();
			print $fh2 $newsline, "\n";

		}

		close($fh);
		close($fh2);

		EditNewsdat_Finish();
	}
}
END_SUB


# Make sure any profiles containing the old Category gets build as well.
# Have to use the "my $variable"-approach since $i is declared localy in BuildNews
my $ChangeDateNew_SpeedupCode = <<'END_CODE';
	if($ChangeDateNew_OldCategory && $ProfCats{$i}->{$ChangeDateNew_OldCategory}) {
		last BUILDNEWS_SKIPDELETE;
	}
END_CODE

$addon->hook('BuildNews_Speedup_Code_Delete', \$ChangeDateNew_SpeedupCode, 10);




# This sub is needed in order to handle the scenario of someone
# entering/altering the End Date to a historical date at the 
# submit/modify news page. Otherwise the newsitem would be
# "inbetween" two Categories during that build and not be
# included in any profile. Now the affect is that the newsitem
# is visable in both Categories for one build ...
$Subs{'ChangeDateNew_InitActiveProfiles'} = <<'END_SUB';
sub ChangeDateNew_InitActiveProfiles {

	# PARAHEAD: Make sure to use the CurrentTime with an offset at all places instead (3.5.2)
	$CurrentTimeOffset = ($CConfig{'TimeOffset'}) ? $CurrentTime + (3600 * $CConfig{'TimeOffset'}) : $CurrentTime;
	
#	if($in{'ChangeDateNew_ValidToDate'})
	if($ChangeDateNew_ValidToDate)
	{
		# Extract the Month_Day, Month and Year the user has specified and get a timestamp
#		my($CD_Year, $CD_Month, $CD_Month_Day) = ChangeDateNew_SplitDate($in{'ChangeDateNew_ValidToDate'});

	    # Get the timesamp for that day
	    # Since we would like the news to be valid *until* the specified day, we add an extra day
#		my $ValidToTimestamp = YMDtoUNIX($CD_Year, $CD_Month, $CD_Month_Day) + (60*60*24);
		
		# PARAHEAD: Fixed a typo, had missed the prefix (3.5.2)
#		if ( $ValidToDate < $CurrentTime )
		if ( $ChangeDateNew_ValidToDate < $CurrentTimeOffset )
		{
			my $newCat = $newscategories{$in{'Category'}}->{'changedatenew_cat'};
			if($newCat && ($newCat ne 'cdn_maintain')) {
				$ChangeDateNew_OldCategory = $in{'Category'};
				$in{'Category'} = $newCat;
				$in{'ChangeDateNew_ValidToDate'} = '';
			}
		}
	}

}
END_SUB

$addon->hook('InitActiveProfiles', 'ChangeDateNew_InitActiveProfiles', 10);



$Subs{'ChangeDateNew_Build_GetData'} = <<'END_SUB';
sub ChangeDateNew_Build_GetData {

	if($ChangeDateNew_ValidToDate)
	{
		# Extract the Month_Day, Month and Year the user has specified and get a timestamp
#		my($CD_Year, $CD_Month, $CD_Month_Day) = ChangeDateNew_SplitDate($ChangeDateNew_ValidToDate);

		
#	my ($Second,$Minute,$Hour,$Month_Day, $Month,$Year,$Week_Day,$IsDST) = localtime($newstime);	
#print "newsid: $newsid, newstime: $newstime, Month: $Month<br>\n";
#print "newsid: $newsid, newstime: $newstime, Month: $CD_Month, message: $CD_Message<br>\n";
			
	    # Get the timesamp for that day
	    # Since we would like the news to be valid *until* the specified day, we add an extra day
#		$ChangeDateNew_ValidToTimestamp = YMDtoUNIX($CD_Year, $CD_Month, $CD_Month_Day) + (60*60*24);
#		$ChangeDateNew_ValidToTimestamp = $ChangeDateNew_ValidToDate + (60*60*24);

#		if ( $ChangeDateNew_ValidToTimestamp < $CurrentTime )
		if ( ($ChangeDateNew_ValidToDate + (60*60*24)) < $CurrentTimeOffset )
		{
			my $newCat = $newscategories{$Category}->{'changedatenew_cat'};
			if($newCat && ($newCat ne 'cdn_maintain')) {
				$ChangeDateNew_EndDateItems{$newsid} = 1;
				$Category = $newCat;
				$ChangeDateNew_ValidToDate = '';
#				$ChangeDateNew_ValidToTimestamp = '';
			}
		}
	}

}
END_SUB

$addon->hook('Build_GetData', 'ChangeDateNew_Build_GetData', 10);

			
			
my $ChangeDateNew_FuturePost = <<'END_CODE';
#print "FuturePost: ChangeDateNew_ValidToDate: $ChangeDateNew_ValidToDate<br>\n";
	# PARAHEAD: This is only needed if we dont loop through the newsdatabase once first... (3.5)
	if($ChangeDateNew_ValidToDate =~ /-/) {
		my ($message, $yyyy, $mm, $dd) = ChangeDateNew_VerifyDate($ChangeDateNew_ValidToDate);
		CRcough("The newsitem '$Subject' (id: $newsid) contains an invalid enddate. $message") if $message;

		eval {
			$ChangeDateNew_ValidToDate = YMDtoUNIX($yyyy, $mm, $dd);
		};
		if ($@) {
			$addon->minorError("The ChangeDateNew addon (in sub ChangeDateNew_FuturePost) had problems with converting ($yyyy, $mm, $dd) via YMDtoUNIX: $@");
		}
	}

#	if ( ($newstime > $CurrentTime && ($CConfig{'ChangeDateNew_FuturePost'} == 1)) || ($ChangeDateNew_ValidToDate && ChangeDateNew_SelectProfs($i) && ($ChangeDateNew_ValidToTimestamp < $CurrentTime)) )
	if ( ($newstime > $CurrentTimeOffset && ($CConfig{'ChangeDateNew_FuturePost'} == 1)) || ($ChangeDateNew_ValidToDate && ChangeDateNew_SelectProfs($i) && ($ChangeDateNew_ValidToDate < $CurrentTimeOffset)) )
	{
		next PROFLOOP;
	}
	else {
		# Do nothing
	}
END_CODE
$addon->hook('BuildNews_Filtering2', \$ChangeDateNew_FuturePost, 10);


# PARAHEAD: Take care of the update of ValidToDates that are stored in the old format (3.5)
$addon->hook('EarlyHook', \<<'END_CODE', 10);
	my $current_CDN = ($CConfig{ChangeDateNew_setup} =~ /^successful: (\d+)$/)?$1:0;
#$addon->simplePage('ChangeDateNew Version', "ChangeDateNew current build: $current_CDN", 1);
#exit;

	#unless ($CConfig{ChangeDateNew_setup} =~ /^successful: (\d+)$/ && $1 == $ChangeDateNew_build) {
	unless ($current_CDN == $ChangeDateNew_build) {

		# At build 35 we changed from saving the ValidToDate in dd-mm-yyy format to Unix timestamp
		if($current_CDN >= 35 && $current_CDN < $ChangeDateNew_build ) {
			$CConfig{'ChangeDateNew_setup'} = "successful: $ChangeDateNew_build";
		} elsif($current_CDN < 35) {
			CRcough("You must be admin to initialize the ChangeDateNew addon") unless($up == 3);
	
			NeedFile('cradmin.pl');
			AreYouSure("ChangeDateNew will now update the news database, to be on the safe side you should <strong>make a backup of the file newsdat.txt</strong> before you continue! Are you sure you want to proceed?") unless $in{'really'};
			# PARAHEAD: Added call to NeedCFG since the SplitDataFile is located there (3.5.1)
			NeedCFG();

			my ($fh, $fh2) = EditNewsdat_Start();
			while (<$fh>) {
				# Loop through items.
				chomp $_;
				SplitDataFile($_);
				if($ChangeDateNew_ValidToDate)
				{
					my ($dd, $mm, $yyyy) = split("-",$ChangeDateNew_ValidToDate);
					eval {
						$ChangeDateNew_ValidToDate = YMDtoUNIX($yyyy, $mm, $dd) if($mm>0);
					};
					if ($@) {
						CRcough("ChangeDateNew had problems upgrading newsdat.txt when converting ($yyyy, $mm, $dd) via YMDtoUNIX: $@");
					}
				}
				my $newsline = JoinDataFile();
				print $fh2 $newsline, "\n";
			}

			close($fh);
			close($fh2);

			EditNewsdat_Finish();
			$CConfig{'ChangeDateNew_setup'} = "successful: $ChangeDateNew_build";
			$addon->simplePage('ChangeDateNew Updated', 'ChangeDateNew has now succesfully updated the news database.', 1);
			exit;
		}
	}
	
END_CODE
	


#$Subs{'CDN_XMS_EditProfile'} = <<'END_SUB';
#sub CDN_XMS_EditProfile {
#	print MidHeading('CDN Heading');
#}
#END_SUB
#
#$addon->hook('XMS_EditProfile', 'CDN_XMS_EditProfile', 10);

1;



__END__

=head1 Change Date New

=head2 INTRODUCTION

This addon makes it possible to alter the submit date and time for a newsitem.
It also makes it possible to specify an end date so that when this date has 
passed the newsitem is filtered out when a build is done. It includes support
for the X Multisubmit addon so that you can specify which submit pages the 
option to alter the newsitems date/time should appear on.

=head2 SUBMISSION DATE OPTIONS

When you specify a new date and time for a newsitem it is the real submit 
timestamp that is altered, so be aware that you will *not* be able to retrieve 
the actual date/time you posted the newsitem, since it will in all aspects 
look like it was submitted at the point in time you changed it to. 
This will also be reflected at the Modify News page in that it will 
be placed at the correct cronological position.

At Administration -> Change Settings you have a couple of ChangeDateNew settings,
where you can specify if news posted in the future should be filtered out or not.
This may come in handy if you would like to post newsarticles ahead of time but
they should not appear until that date has passed. The other scenario would
be that you post info about upcoming events which you do like to present.

You can also choose to only alter the submission date but not the time. Some times
you are not interested in the actual time of day the newsitem was submitted, just
that it was submitted a specific day. If you have this option enabled and change
the submission date at the Submit News page, the time will be set to '00:00:01'.

=head2 VALID TO DATE OPTIONS

The Valid To/End Date feature is optional, you can turn off the field from
appearing at the Submit News page all together. But if you have it enabled
it is also possible to choose which profiles it should be active for,
since you can use different profiles to achive different effects.
An example would be if you have created one profile for upcoming events,
and another profile that is set up to show only the historical events.
Only the upcoming events profile should be activated in this case.

If you have the Category addon enabled, you will also be able to specify
on a per Category basis if a newsitem posted to a certain Category
should be transfered to a new Category when the End Date has been reached.
This is done under Administration -> News Categories -> "Edit Permissions"
for each Category. 

The value of the End Date field is accessible in a style through the tag
<Field: ChangeDateNew_ValidToDate>.

=head2 AUTOBUILD AND CRON

Since ChangeDateNew lets you specify
other submission dates and valid to dates it can be needed to do a full
rebuild periodically, for example once every night, to make sure
that newsitems that you have posted ahead of time shows up at the specific
date and of course newsitems that should be removed from the visual output
actually is removed.

The solution is to use cron, which is the name of a program that enables
Unix users to execute commands or scripts automatically at a
specified date and time or in intervalls without user
intervention every time, for example each night at 3 AM.

In previous distributions of ChangeDateNew a specific script to perform
the full rebuild (activated through cron) was included, but since
there allready exists an addon called Autobuild which was
created for this specific task I have decided not to include the CDN build
script in this release and would suggest using the Autobuild addon instead.
A download link for Autobuild can be found at CTUS (L<http://coranto.gweilo.org/>).
The documentation included with that addon gives you directions of how to
set it up, but if you run into problems, you could try
using one of the following commands; I<curl>, I<wget> or I<fetch>.

Below is an example of how to use curl:
=begin html
<pre>
curl -s http://www.yourdomain.com/cgi-bin/coranto/autobuild.cgi?full
</pre>
=end html

Below is an example of how to use wget:
=begin html
<pre>
wget -q http://your.server.name.here/cgi-bin/coranto/autobuild.cgi?full
</pre>
=end html

Below is an example of how to use fetch:
=begin html
<pre>
fetch http://your.server.name.here/cgi-bin/coranto/autobuild.cgi?full 
</pre>
=end html

Do a search for 'curl' in the forum at CTUS and you will find some threads
discussing this topic further.

=head2 ABOUT THE AUTHORS

This addon was originally coded by Lawrence, all credits goes to him!
This current version is modified by me, Parahead, and released with
Lawrence approval, but if he decide to continue development of this
addon I surrender all the updates I have made to this addon
to him and he may do what he pleases with them.

=over

=item *

name: Henrik Ahln aka Parahead

=item *

email: L<mailto:henrik@parahead.com>

=item *

site: L<http://www.parahead.com/>

=back

=head2 VERSION HISTORY

=head3 3.5.6 by Parahead (2005-03-03)

=over

=item *

Bugfix, hooks inserted into the NewsCategory addon now done through EarlyHook instead
of when the addon loads, which could cause problem if CDN was loaded before ncategory.

=item *

Bugfix, changed the sub ChangeDateNew_FuturePost to a 'my-sub' instead,
since it needs access to the profile name when calling ChangeDateNew_SelectProfs.

=item *

Bugfix, the subs ChangeDateNew_SelectProfs and ChangeDateNew_ProfSelect now handles
the virtual headline profiles correctly, that is using the main profile settings.

=back

=head3 3.5.5 by Parahead (2005-02-19)

=over

=item *

Bugfix, posting a newsitem without changing the submit date/time when a newsitem ahead of time
already existed made the current newsitem appear before the fresher one.

=back

=head3 3.5.4 by Parahead (2005-02-12)

=over

=item *

Bugfix, if only changing the submission time the date was set back to 1970-01-01 (year zero in Unix)

=back

=head3 3.5.3 by Parahead (2005-02-12)

=over

=item *

Bugfix, previously the hook inserted into the NewsCategories_EditCat sub 
within the Category addon only worked for the build bundled together with Coranto 1.30.x
(Used for the specification of which category to move a newsitem to when the end date was reached)

=item *

Included a section about Autobuild and cron in the documentation

=back

=head3 3.5.2 by Parahead (2005-02-05)

=over

=item *

Bugfix, now uses the time offset setting correctly when filtering newsitems (both future dates and valid to dates)

=item *

Bugfix, a small typo (in the sub ChangeDateNew_InitActiveProfiles) preventing a modification
of the Valid To Date to a past date to work as intended 

=back

=head3 3.5.1 by Parahead (2005-02-05)

=over

=item *

Bugfix, added call to NeedCFG when upgrading from old version of the addon

=item *

Improved the build check when upgrading from old version of the addon

=back

=head3 3.5 by Parahead (2005-02-02)

=over

=item *

Improved the error checks and corresponding messages.

=item *

Now possible to choose if both date and time (or only the date) should be possible to alter.

=item *

If the 'only alter date' option is choosen, the time is set to '00:00:01' if the date is changed.

=item *

Added option to choose which date format to use when entering dates at the submit news page.

=item *

The news database is updated when enabling this version to use Unix timestamp instead of 'dd-mm-yyyy'.

=item *

Changed marker in ChangeDateNew_SpeedupCode sub to BUILDNEWS_SKIPDELETE to comply with the 1.30.x series of Coranto.

=item *

Changed the precedence for when the hook SaveNews_2 and ChangeDateNew_SaveNews_2 should be executed to -10
(Reference thread: L<http://coranto.gweilo.org/forum/viewtopic.php?t=6776>).

=back

=head3 3.4 by Parahead (2004-07-30)

=over

=item *

Bug removed in sub ChangeDateNew_SaveNews_2 that made a newsitem not being sorted correctly at the Modify News page if only the time was changed.

=item *

Added support for moving a newsitem to another Cateogory when the End Date has been reached.

=item *

Added documentation accessible via the Addon Manager (eg. this document!)

=back

=head3 3.3 by Parahead (2004-06-05)

=over

=item *

The effort to try and comply with how the countnews array worked made things worse, so now the decrement is removed.

=back

=head3 3.2 by Parahead (2004-04-13)

=over

=item *

Now possible to specify which profiles the End Date should be active for. 

=item *

Decrement of the countnews array is now done

=back

=head3 3.1 by Parahead (2004-01-24)

=over

=item *

First release by Parahead

=item *

An End Date feature has been added. Now you can specify an end date so that the newsitem will be filtered out after that date when doing a build. The value of the field is accessible in the style via the tag <Field: ChangeDateNew_ValidToDate>.

=item *

Support for the X Multisubmit addon added, now you can restrict presentation of the ChangeDateNew fields to the XMS pages of your choice. No need for the XMS addon to be installed but if it is at least version 2.0 beta 7 is required.

=item *

The actual filtering of the newsitem has been moved from the hook Build_GetData to the News_Filtering2 hook instead, to better comply with the functionality of how the countnews array works.

=back

